EvaluateScript("flik_taginfo.js");

var cpl_current_song = 0;
var cpl_next_song = 0;
var cpl_files = undefined;
var cpl_playlist = undefined;
var cpl_current_index = 0;
var cpl_song_count = 0;
var cpl_repeat = 0;
var cpl_shuffle = true;
var cpl_playing = false;
var cpl_paused = false;
var cpl_muted = false;

var cpl_song_info = undefined;

function GetShuffledSong(){
	var cpl_random = Math.floor(Math.random() * (cpl_song_count - 1));
	if(cpl_random >= cpl_current_song){
		cpl_random += 1;
	}
	return cpl_random;
}

function CreatePlaylist(root, streaming){
	if(arguments.length == 1){
		streaming = false;
	}
	cpl_files = GetFileList(root);
	cpl_playlist = new Array(cpl_files.length);
	cpl_song_info = new Array(cpl_files.length);
	cpl_current_index = 0;
	for(var cpl_a = 0; cpl_a < cpl_files.length; cpl_a++){
		//if(cpl_files[cpl_a].toLowerCase().slice(cpl_files.length - 4) == ".mp3"){
		var ext = cpl_files[cpl_a].toLowerCase().slice(cpl_files[cpl_a].length - 4);
		if(ext == ".mp3" || ext == ".ogg"){
			cpl_playlist[cpl_current_index++] = LoadSound(cpl_files[cpl_a], streaming);
			cpl_song_info[cpl_a] = ReadTagInfo(cpl_files[cpl_a]);
			var cpl_length = cpl_playlist[cpl_a].getLength();
			var cpl_milliseconds = cpl_length / 44.1;
			cpl_song_info[cpl_a].minutes = Math.floor(cpl_milliseconds / 60000);
			cpl_song_info[cpl_a].seconds = Math.floor((cpl_milliseconds - cpl_song_info[cpl_a].minutes * 60000) / 1000);
		}
	}
	cpl_song_count = cpl_playlist.length;
}

function PlaySong(song){
	if(cpl_playlist[cpl_current_song].isPlaying() == false){
		cpl_playing = true;
		cpl_paused = false;
		cpl_playlist[cpl_current_song = song].play(cpl_repeat == -1 ? true : false);
	}
}

function StopSong(){
	if(cpl_playlist[cpl_current_song].isPlaying() == true){
		cpl_playing = false;
		cpl_paused = true;
		cpl_playlist[cpl_current_song].stop();
	}
}

function PauseSong(){
	cpl_paused = true;
	cpl_playlist[cpl_current_song].pause();
}

function UnPauseSong(){
	cpl_paused = false;
	cpl_playlist[cpl_current_song].play(cpl_repeat == -1 ? true : false);
}

function StartPlaylist(position){
	cpl_current_song = 0;
	cpl_current_song = Math.floor(Math.random() * cpl_song_count);
	if(position == NaN){
		cpl_current_song = position % cpl_song_count;
	}		
	PlaySong(cpl_current_song);
}

function SkipToSong(song, immediate){
	while(song < 0){
		song += cpl_song_count;
	}
	cpl_next_song = song % cpl_song_count;
	if(immediate == true){
		StopSong();
		PlaySong(cpl_next_song);
	}
}

function NextSong(){
	SkipToSong(cpl_current_song + 1, true);
}

function PreviousSong(){
	SkipToSong(cpl_current_song - 1, true);
}

function UpdatePlaylist(){
	if(cpl_playlist[cpl_current_song].isPlaying() == false && cpl_playing == true && cpl_paused == false){
		if(cpl_shuffle == true){
			SkipToSong(GetShuffledSong(), true);
		}
		else{
			NextSong();
		}
	}
}

function SetSongVolume(){
	if(arguments.length == 1){
		for(var cpl_b = 0; cpl_b < cpl_playlist.length; cpl_b++){
			if(arguments[cpl_b] == NaN){
				Abort("Incorrect call to SetSongVolume(), all parameters must be intergers");
			}
			cpl_playlist[cpl_b].setVolume(arguments[0]);
		}
	}
	else if(arguments.length % 2 != 0){
		for(var cpl_c = 0; cpl_c < arguments.length / 2; cpl_c++){
			if(arguments[cpl_c * 2] == NaN){
				Abort("Incorrect call to SetSongVolume(), all parameters must be intergers");
			}
			if(arguments[(cpl_c * 2) + 1] == NaN){
				Abort("Incorrect call to SetSongVolume(), all parameters must be intergers");
			}
			cpl_playlist[cpl_c * 2].setVolume(arguments[(cpl_c * 2) + 1]);
		}
	}
	else{
		Abort("Incorrect call to SetSongVolume(), SetSongVolume() must be called with either one parameter, or a number of parameters which is divisible by two");
	}
}

function GetSongVolume(){
	if(arguments.length == 1){
		if(arguments[0] == NaN){
			Abort("Incorrect call to GetSongVolume(), all parameters must be intergers");
		}
		return cpl_playlist[arguments[0]].getVolume();
	}
	else if(arguments.length == 0){
		Abort("Incorrect call to GetSongVolume(), GetSongVolume() requires at least one parameter");
	}
	else{
		var cpl_volume_return = new Array(arguments.length);
		for(var cpl_d = 0; cpl_d < arguments.length; cpl_d++){
			if(arguments[cpl_d] == NaN){
				Abort("Incorrect call to GetSongVolume(), all parameters must be intergers");
			}
			cpl_volume_return[cpl_d] = cpl_playlist[arguments[cpl_d]].getVolume();
		}
		return cpl_volume_return;
	}
}

function SetSongPan(){
	if(arguments.length == 1){
		for(var cpl_e = 0; cpl_e < cpl_playlist.length; cpl_e++){
			if(arguments[cpl_e] == NaN){
				Abort("Incorrect call to SetSongPan(), all parameters must be intergers");
			}
			cpl_playlist[cpl_e].setPan(arguments[0]);
		}
	}
	else if(arguments.length % 2 != 0){
		for(var cpl_f = 0; cpl_f < arguments.length / 2; cpl_f++){
			if(arguments[cpl_f * 2] == NaN){
				Abort("Incorrect call to SetSongPan(), all parameters must be intergers");
			}
			if(arguments[(cpl_f * 2) + 1] == NaN){
				Abort("Incorrect call to SetSongPan(), all parameters must be intergers");
			}
			cpl_playlist[cpl_f * 2].setPan(arguments[(cpl_f * 2) + 1]);
		}
	}
	else{
		Abort("Incorrect call to SetSongPan(), SetSongPan() must be called with either one parameter, or a number of parameters which is divisible by two");
	}
}

function GetSongPan(){
	if(arguments.length == 1){
		if(arguments[0] == NaN){
			Abort("Incorrect call to GetSongPan(), all parameters must be intergers");
		}
		return cpl_playlist[arguments[0]].getPan();
	}
	else if(arguments.length == 0){
		Abort("Incorrect call to GetSongPan(), GetSongPan() requires at least one parameter");
	}
	else{
		var cpl_pan_return = new Array(arguments.length);
		for(var cpl_g = 0; cpl_g < arguments.length; cpl_g++){
			if(arguments[cpl_g] == NaN){
				Abort("Incorrect call to GetSongPan(), all parameters must be intergers");
			}
			cpl_pan_return[cpl_g] = cpl_playlist[arguments[cpl_g]].getPan();
		}
		return cpl_pan_return;
	}
}

function SetSongPitch(){
	if(arguments.length == 1){
		for(var cpl_h = 0; cpl_h < cpl_playlist.length; cpl_h++){
			if(arguments[cpl_h] == NaN){
				Abort("Incorrect call to SetSongPitch(), all parameters must be intergers");
			}
			cpl_playlist[cpl_h].setPitch(arguments[0]);
		}
	}
	else if(arguments.length % 2 != 0){
		for(var cpl_i = 0; cpl_i < arguments.length / 2; cpl_i++){
			if(arguments[cpl_i * 2] == NaN){
				Abort("Incorrect call to SetSongPitch(), all parameters must be intergers");
			}
			if(arguments[(cpl_i * 2) + 1] == NaN){
				Abort("Incorrect call to SetSongPitch(), all parameters must be intergers");
			}
			cpl_playlist[cpl_i * 2].setPitch(arguments[(cpl_i * 2) + 1]);
		}
	}
	else{
		Abort("Incorrect call to SetSongPitch(), SetSongPitch() must be called with either one parameter, or a number of parameters which is divisible by two");
	}
}

function GetSongPitch(){
	if(arguments.length == 1){
		if(arguments[0] == NaN){
			Abort("Incorrect call to GetSongPitch(), all parameters must be intergers");
		}
		return cpl_playlist[arguments[0]].getPitch();
	}
	else if(arguments.length == 0){
		Abort("Incorrect call to GetSongPitch(), GetSongPitch() requires at least one parameter");
	}
	else{
		var cpl_pitch_return = new Array(arguments.length);
		for(var cpl_j = 0; cpl_j < arguments.length; cpl_j++){
			if(arguments[cpl_j] == NaN){
				Abort("Incorrect call to GetSongPitch(), all parameters must be intergers");
			}
			cpl_pitch_return[cpl_j] = cpl_playlist[arguments[cpl_j]].getPitch();
		}
		return cpl_pitch_return;
	}
}

function SetSongPosition(song, position){
	if(arguments.length != 2){
		Abort("Incorrect call to SetSongPosition(), SetSongPosition() requires exactly two parameters");
	}
	if(arguments[0] == NaN){
		Abort("Incorrect call to SetSongPosition(), all parameters must be intergers");
	}
	if(arguments[1] == NaN){
		Abort("Incorrect call to SetSongPosition(), all parameters must be intergers");
	}
	if(cpl_playlist[cpl_current_song].isSeekable() == true){
		var cpl_song_length = cpl_playlist[cpl_current_song].getLength();
		cpl_playlist[cpl_current_song].setPosition(position % cpl_song_length);
		return true;
	}
	return false;
}

function GetSongPosition(song){
	if(arguments.length != 1){
		Abort("Incorrect call to GetSongPosition(), GetSongPosition() requires exactly one parameter");
	}
	if(arguments[0] == NaN){
		Abort("Incorrect call to GetSongPosition(), all parameters must be intergers");
	}
	if(cpl_playlist[cpl_current_song].isSeekable() == true){
		return cpl_playlist[cpl_current_song].getPosition();
	}
	return false;
}

function GetSongLength(){
	if(arguments.length == 1){
		if(arguments[0] == NaN){
			Abort("Incorrect call to GetSongLength(), all parameters must be intergers");
		}
		return cpl_playlist[arguments[0]].getLength();
	}
	else if(arguments.length == 0){
		Abort("Incorrect call to GetSongLength(), GetSongLength() requires at least one parameter");
	}
	else{
		var cpl_length_return = new Array(arguments.length);
		for(var cpl_k = 0; cpl_k < arguments.length; cpl_k++){
			if(arguments[cpl_k] == NaN){
				Abort("Incorrect call to GetSongLength(), all parameters must be intergers");
			}
			cpl_length_return[cpl_k] = cpl_playlist[arguments[cpl_k]].getLength();
		}
		return cpl_length_return;
	}
}

function MutePlaylist(){
	cpl_muted = true;
}

function UnMutePlaylist(){
	cpl_muted = false;
}