///////////////////////////////////////////////////////////

/**
  this returns the equilivent TagInfo object (MP3v1TagInfo or OggTagInfo)
  based on the signiture of the filename given, returns null on error
*/

var dead_object = new Object();
  dead_object.title = "Untitled";
  dead_object.artist = "Unknown";
  dead_object.album = "Unknown";
  dead_object.year = 1999;
  dead_object.genre = "None";
  dead_object.comment = "Could not read tag";
  dead_object.trackNumber = 0;

function ReadTagInfo(filename) {
  var ext = filename.toLowerCase().slice(filename.length - 4);
  switch (ext) {
    case (".mp3"):
      var temp = ReadMP3v1TagInfo(filename);    
    break;

    case (".ogg"):
      var temp = ReadOggTagInfo(filename);    
    break;
  }
  if(temp != null){
		return temp;
	}
	return dead_object;
}

///////////////////////////////////////////////////////////

/**
  some byte arrays can return bad strings when you convert them
  this version of CreateStringFromByteArray is a workaround for it
*/
function __CreateStringFromByteArray__(data) {
  for (var i = 0; i < data.length; i++)
    if (data[i] == 0)
      break;
  if (i > 0)
    return CreateStringFromByteArray(data.slice(0, i));
  return "";
}

///////////////////////////////////////////////////////////

/**
  Some mp3 tags have data padded with spaces
  (instead of being null terminated) this is a fix for that
*/
function FixMP3TagData(data) {
  for (var i = data.length - 1; i >= 0; i--) {
    if ( (data[i] == 0x20) || (data[i] == 0) ) {
      data[i] = 0;
      break;
    }
  }

  return data;
}

///////////////////////////////////////////////////////////

/**
  this converts the genre code in mp3v1 tags to a string, and returns "" on failure
*/
function MP3GenreCodeToString(code) {
  var codes = genre_array = new Array("Blues","Classic Rock","Country","Dance","Disco",
  "Funk","Grunge", "Hip-Hop","Jazz","Metal","New Age","Oldies","Other","Pop","R&B", 
  "Rap","Reggae","Rock","Techno","Industrial","Alternative","Ska", 
  "Death Metal","Pranks","Soundtrack","Euro-Techno","Ambient", 
  "Trip-Hop","Vocal","Jazz+Funk","Fusion","Trance","Classical", 
  "Instrumental","Acid","House","Game","Sound Clip","Gospel", 
  "Noise","AlternRock","Bass","Soul","Punk","Space","Meditative", 
  "Instrumental Pop","Instrumental Rock","Ethnic","Gothic", 
  "Darkwave","Techno-Industrial","Electronic","Pop-Folk", 
  "Eurodance","Dream","Southern Rock","Comedy","Cult","Gangsta", 
  "Top 40","Christian Rap","Pop/Funk","Jungle","Native American", 
  "Cabaret","New Wave","Psychadelic","Rave","Showtunes","Trailer", 
  "Lo-Fi","Tribal","Acid Punk","Acid Jazz","Polka","Retro", 
  "Musical","Rock & Roll","Hard Rock","Folk","Folk-Rock", 
  "National Folk","Swing","Fast Fusion","Bebob","Latin","Revival", 
  "Celtic","Bluegrass","Avantgarde","Gothic Rock","Progressive Rock", 
  "Psychedelic Rock","Symphonic Rock","Slow Rock","Big Band", 
  "Chorus","Easy Listening","Acoustic","Humour","Speech","Chanson", 
  "Opera","Chamber Music","Sonata","Symphony","Booty Bass","Primus", 
  "Porn Groove","Satire","Slow Jam","Club","Tango","Samba", 
  "Folklore","Ballad","Power Ballad","Rhythmic Soul","Freestyle", 
  "Duet","Punk Rock","Drum Solo","Acapella","Euro-House","Dance Hall");

  if (code >= 0 && code < genre_array.length)
    return genre_array[code];
  return "";
}

///////////////////////////////////////////////////////////

/**
  An MP3v1TagInfo object
*/
function MP3v1TagInfo(title, artist, album, year, genre, comment, track_number) {
  this.title = title;
  this.artist = artist;
  this.album = album;
  this.year = year;
  this.genre = MP3GenreCodeToString(genre);
  this.comment = comment;
  this.trackNumber = track_number;

  this.toString = function() {
    return "title=" + title + "\n"
         + "artist=" + artist + "\n"
         + "album=" + album + "\n"
         + "year=" + year + "\n"
         + "genre=" + MP3GenreCodeToString(genre) + "\n"
         + "comment=" + comment + "\n"
         + "track_number=" + track_number + "\n";
  }
}

///////////////////////////////////////////////////////////

/**
  Reads an MP3v1TagInfo object from filename, returning null on error
*/
function ReadMP3v1TagInfo(filename) {
  var file = OpenRawFile("../sounds/" + filename);
  if (file.getSize() < 128)
    return null;
  file.setPosition(file.getSize() - 128);
  var data = file.read(128);
  if (data.length < 128)
    return null;
  
  var is_tag = CreateStringFromByteArray(data.slice(0, 3)) == "TAG";

  if (is_tag) {
    var title  = __CreateStringFromByteArray__(FixMP3TagData(data.slice(3, 3 + 30)));
    var artist = __CreateStringFromByteArray__(FixMP3TagData(data.slice(33, 33 + 30)));
    var album = __CreateStringFromByteArray__(FixMP3TagData(data.slice(63, 63 + 30)));
    var year = __CreateStringFromByteArray__(data.slice(93, 93 + 4));
    var genre = data.slice(127)[0];
    var comment = "";
    var track_number = -1;
    var version = 1.0;

    if ( (data[125] == 0 ) && (data[126] != 0) ) {
      version = 1.1;
    }

    if (version == 1.1) {
      comment = __CreateStringFromByteArray__(FixMP3TagData(data.slice(97, 97 + 28)));
      track_number = data[126];
    } else {
      comment = __CreateStringFromByteArray__(FixMP3TagData(data.slice(97, 97 + 30)));
    }

    return new MP3v1TagInfo(title, artist, album, year, genre, comment, track_number);

  }

  return null;
}

///////////////////////////////////////////////////////////

function read32BitUnsignedInteger(file) {
  var n = 0;
  var t;
  var bytes = file.read(4);

  var first = bytes[0];
  var second = bytes[1];
  var third = bytes[2];
  var fourth = bytes[3];

  t = first;
  t = t << 24;
  n = n | t;
  t = second;
  t = t << 16;
  n = n | t;
  t = third;
  t = t << 8;
  n = n | t;
  t = fourth;
  n = n | t;

  return bytes[0]; // -_-
}

///////////////////////////////////////////////////////////

/**
  OggTagInfo object
*/
function OggTagInfo(user_comment_list) {
  function find_comment(name) {
    var key = name + "=";
    for (var i = 0; i < user_comment_list.length; i++) {
      var field = user_comment_list[i].toLowerCase();
      if (field.indexOf(key) == 0) {
        return user_comment_list[i].slice(key.length);
      }
    }
    return "";
  }

  this.getTitle = function() { return find_comment("title"); }
  this.getArtist = function() { return find_comment("artist"); }
  this.getAlbum = function() { return find_comment("album"); }
  this.getYear = function() { return find_comment("year"); }
  this.getGenre = function() { return find_comment("genre"); }
  this.getComment = function() { return find_comment("comment"); }
  this.getTrackNumber = function() { return find_comment("tracknumber"); }

  this.toString = function() {  
    function join(list, seperator, terminator) {
      var str = "";
      for (var i = 0; i < list.length; i++) {
        str += list[i];
        if (i < list.length - 1) {
          str += terminator;
        }
      }
      return str;
    }
  
    return join(user_comment_list, "=", "\n");
  }
}

///////////////////////////////////////////////////////////

/**
  Reads an OggTagInfo object from filename, returning null on error
*/
function ReadOggTagInfo(filename) {
  var file = OpenRawFile("../sounds/" + filename);
  if (file.getSize() < 200)
    return null;
  var start = 103;
      file.setPosition(start);

  var is_vorbis = CreateStringFromByteArray(file.read(6)) == "vorbis";
  
  if (is_vorbis) {
    var vendor_length = read32BitUnsignedInteger(file);
    var vendor_name = __CreateStringFromByteArray__(file.read(vendor_length));
    var user_comment_length = read32BitUnsignedInteger(file);
    var user_comment_list = new Array();

    for (var i = 0; i < user_comment_length; i++) {
      var length = read32BitUnsignedInteger(file);
      var str = CreateStringFromByteArray(file.read(length));
      user_comment_list.push(str);
    }

    var frame_bit_is_set = file.read(1)[0] & 0xFF;
    if (!frame_bit_is_set)
      return null;
    
    return new OggTagInfo(user_comment_list);  
  }

  return null;
}

///////////////////////////////////////////////////////////
